/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

// lib.cc
// alex iliev jan 2003
// some shared routines for the card

#include <string>
#include <iostream>
#include <fstream>

#include <stdio.h>

#include <common/utils.h>
#include <common/sym_crypto.h>
#include <common/consts.h>

#include "lib.h"


using namespace std;


extern const char SHUFFLE_FILE[] = "shuffle.txt";

extern const size_t BUCKETSIZE = 5;
extern const size_t DBSIZE = 1000;


void
loadkeys (byte * o_key, const string& keyfile,
	  byte * o_mackey, const string& mackeyfile)
    throw (ios::failure)
{

    byte *keys[] = { o_key, o_mackey };
    const char *files[] = {keyfile.c_str(), mackeyfile.c_str() };

    for (int i = 0; i < 2; i++) {
	FILE * fh = fopen (files[i], "r");
	if (!fh) {
	    throw ios::failure
		(string ("Error opening key file: ") + strerror(errno));
	}

	fread (keys[i], 1, DES3_KEY_SIZE, fh);

	fclose (fh);
    }
}
    


// PRE: V is empty
void load_shuffle (vector<index_t>& V, const string& filename)
    throw (ios::failure)
{
    ifstream fis (filename.c_str());

    fis.exceptions (ifstream::badbit);
    
    string s;
    while (getline (fis, s)) {
	V.push_back (atoi(s.c_str()));
    }
}
    

string get_shuffle_filename () {
    return string (STOREROOT) + "/" + SHUFFLE_FILE;
}


ByteBuffer addmac (const ByteBuffer& ciphertext, const ByteBuffer& mac) {

    size_t length = ciphertext.len() + mac.len();

    ByteBuffer answer (new byte [length], length);

    memcpy (answer.data(), ciphertext.data(), ciphertext.len());
    memcpy (answer.data() + ciphertext.len(), mac.data(), mac.len());

    return answer;
}


// NOTE: this does not allocate new memory but aliases into the input buffer.
// so, do not use the return value longer than the input buffer exists
ByteBuffer getmac (const ByteBuffer& buf) {

    // the MAC is the last 8 bytes of the buffer (for DES MAC)

    return ByteBuffer (buf.data() + (buf.len()-DES_MAC_SIZE),
		       DES_MAC_SIZE,
		       ByteBuffer::no_free);
}

// same comment as above
ByteBuffer getcipher (const ByteBuffer& buf) {

    return ByteBuffer (buf.data(), buf.len()-DES_MAC_SIZE, ByteBuffer::no_free);

}
