/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/* -*- c++ -*-
 * pirsession.h
 * alex iliev, nov 2002
 * the PIRSession class
 */


#ifndef _PIRSESSION_H
#define _PIRSESSION_H


#include <list>

#include <common/exceptions.h>
#include <common/record.h>
#include <common/sym_crypto.h>



class Bucket;


/*
 * Class to represent a session, between two database shuffles
 */
class PIRSession {

public:

    PIRSession (const ByteBuffer & key, const ByteBuffer & mackey,
		const std::vector<index_t> * shuffle,
		size_t bucket_size);


    /// retrieve a record with the given name.
    Record
    find_record (const std::string& name)
	throw (host_exception, crypto_exception);

    // retrieve a numbered record
    Record
    find_record (index_t number) throw ();

    

#if 0
    // write a record to the host, after padding, encrypting and maccing
    void
    write_record (const Record& rec, const record_id& where);
#endif

    
private:
    

    SymDencrypter _decrypter;
    MacExpert _mac_checker;

    std::vector<record_id> _touched_records; // we'll store each bucket and
                                             // record number retrieved
    
    const std::vector<index_t> * _shuffle; // how are the records currently
				           // shuffled?

    size_t _bucket_size;	// how many records per bucket?

    //
    // helpers
    //

    // fetch a whole bucket, and keep the named record, write into o_enc_rec
    void read_bucket (encrypted_record & o_enc_rec, const record_id& rid)
	throw (host_exception);
    
    // retrieve a named record from a given bucket
    // also return the record number in that bucket, in io_id.record
    void find_record_from_bucket (Record & o_rec,
				  record_id & io_id, const std::string& name)
	throw (crypto_exception, host_exception);


    // get a random record, returning both the record and its ID
    void fetch_random_record (record_id & o_id,
			      Record & o_rec)
	throw (crypto_exception, host_exception);


    // quickie to fetch an exact record
    void fetch_record (Record & o_rec, const record_id& id)
	throw (crypto_exception, host_exception);
    

    // may throw if the MAC doesnt pass
    Record decrypt_record (const ByteBuffer& hostbytes)
	throw (crypto_exception);
    
};


//
// a Bucket class, main utility is to deal with deallocation
//

class Bucket {

public:

    std::list<Record> records;	// access freely!

};


#endif // _PIRSESSION_H
