/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/* -*-c++-*-
 * shuffle.h
 * alex iliev, jan 2003
 * the database shuffling code
 */


#ifndef _SHUFFLE_H
#define _SHUFFLE_H


#include <vector>

#include <stdlib.h>

#include <common/utils.h>
#include <common/exceptions.h>
#include <common/sym_crypto.h>

#include "pirsession.h"		// for Bucket


class Shuffler {

public:

    Shuffler (const ByteBuffer & key, const ByteBuffer & mackey,
	      size_t N, size_t bucket_size);
    
    void generate_shuffle ();

    void write_shuffle (const std::string& filename)
	throw (std::ios::failure);

    void shuffle ()
	throw (host_exception, crypto_exception);


private:

    ByteBuffer _key, _mackey;
    SymDencrypter _enc;
    MacExpert _mac;
    
    std::vector<index_t> V;
    size_t N;
    size_t _bucket_size;
    
    
    //
    // helpers
    //
    
    // fetch a cleartext bucket
    void fetch_bucket (index_t n, Bucket & o_records, size_t bucket_size)
	throw (host_exception);

    // encrypt and write a bucket
    void write_bucket (index_t n, const Bucket & bucket)
	throw (host_exception, crypto_exception);

    

    void write_shuffle (const std::vector<index_t>& V)
	throw (std::ios::failure);
    
};


// set V_[V[i]] = i for all indices i
void transpose_vector (std::vector<index_t> & V_,
		       const std::vector<index_t>& V);


#endif // _SHUFFLE_H
