/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * card_server.cc
 * alex iliev, jan 2003
 * server to service requests from the card, eg to read and write records
 */

#include <iostream>

#include <unistd.h>

#include <stdlib.h>

#include <common/consts.h>
#include <common/sccutils.h>
#include <common/utils.h>

#include "hostcall.h"



using namespace std;



static void usage (const char* progname);
static void reqloop (int sock);


int main (int argc, char *argv[]) {

    unsigned short port = PIR_HOSTSERV_PORT;
    int sock;


    int opt;
    while((opt = getopt(argc, argv, "p:")) != EOF) {
	switch(opt) {
	case('p'):
	    port = atoi(optarg); break;
	    
	default:
	    usage(argv[0]);
	    exit(EXIT_FAILURE); break;
	} /* switch options */
    }

	
    //
    // set up an SCC socket and loop waiting for requests
    //

    sock = scc_setup_server_sock (port);

    reqloop (sock);
}


static void usage (const char* progname) {
    cerr << "Usage: " << progname << ": [-p port]" << endl;
}


static void reqloop (int sock) {

    byte reqbuf[BUFSIZE];

    struct sockaddr_scc scc;
    socklen_t scclen;
    int rc;
    
    while(true) {

	/* get request */
    	scclen = sizeof(scc);
	if ((rc = recvfrom
	     (sock, reqbuf, BUFSIZE, 0,
	      reinterpret_cast<struct sockaddr *>(&scc), &scclen)) < 0)
	{
	    perror ("Receiving request");
	    close (sock);
	    exit (EXIT_FAILURE);
	}

	if (rc == static_cast<int>(BUFSIZE)) {
	    // truncation??
	    // TODO
	}

	ByteBuffer req (reqbuf, rc, ByteBuffer::no_free);

	ByteBuffer answer = host_handle_request (req);
	
	if ( (rc = sendto
	      (sock, answer.data(), answer.len(), 0,
	       reinterpret_cast<struct sockaddr *>(&scc), scclen))
	     != static_cast<int>(answer.len()) )
	{
	    perror ("Sending response");
	}

    }

}
