/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

// makerecords.cc
// alex iliev jan 2003

// code to make and hash records for the private directory


#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <vector>

#include <stdlib.h>

#include <common/record.h>
#include <common/hash.h>
#include <common/consts.h>


using namespace std;


static const size_t DBSIZE = 1000;

Record make_record (const string& name);

int main () {

    string name;

    vector<index_t> bucket_sizes (DBSIZE, 0);
    size_t max_size = 0;
    
    while (getline (cin, name)) {

	ostringstream filename;

	index_t hashed = hash
	    (reinterpret_cast<const unsigned char*> (name.c_str()), DBSIZE);

	filename << STOREROOT << "/" << "clear" << "/" << hashed;
	system ( ("mkdir -p " + filename.str()).c_str() );

	filename << "/" << bucket_sizes[hashed];
	bucket_sizes[hashed] ++;

	max_size = max (max_size, bucket_sizes[hashed]);

	ofstream ofs (filename.str().c_str());
	if (!ofs) {
	    cerr << "Problem opening " << filename.str() << endl;
	    abort();
	}
	
	ofs << make_record(name);

    }

    //
    // now need to add filler records to each bucket so they all have the same
    // size

    // just make it with a constant name for now - can use more random names
    // later

    clog << "Starting to write filler records: "
	 << max_size << " reocrds per bucket" << endl;
    
    for (index_t i=0; i < DBSIZE; i++) {

	ostringstream dir;
	dir << STOREROOT << "/" << "clear" << "/" << i;
	system ( ("mkdir -p " + dir.str()).c_str() );

	for (index_t j = bucket_sizes[i]; j < max_size; j++) {

	    ostringstream  filename;
	    
	    filename << dir.str() << "/" << j;
	    ofstream ofs (filename.str().c_str());
	    if (!ofs) {
		cerr << "Problem opening " << filename.str() << endl;
		abort();
	    }
	
	    ofs << make_record("Anton Dziribiewski");
	    
	}

    }

}


Record make_record (const string& name) {

    Record r;
    r.name = name;

    ostringstream os;

    r.attributes.push_back (RecordAttr ("cn", name));
    
    os << 2000 + random() % 10000;
    r.attributes.push_back (RecordAttr ("uid", os.str()));

    istringstream is_name (name);
    string givenName, sn, middle;
    is_name >> givenName >> middle >> sn;
    if (sn.length() == 0)
	sn = middle; // didnt have a middle name it seems
    r.attributes.push_back (RecordAttr ("givenName", givenName));
    r.attributes.push_back (RecordAttr ("sn", sn));

    sn[0] = tolower(sn[0]);
    r.attributes.push_back (RecordAttr ("mail", sn+"@dartmoor.com"));

    // make some random looking ASCII strings as pretend certificates
    for (int j = 0; j < 3; j++) {
	os.str(string());
	for (int i=0; i<400; i++) {
	    os << char ('A' + random() % ('Z' - 'A'));
	}
	char certnum[] = {j+'1', '\0'};
	r.attributes.push_back (RecordAttr (string("cert") + certnum,
					    os.str()));
    }
    
    os.str(string());
    os << (12 + random() % 55);
    r.attributes.push_back (RecordAttr ("age", os.str()));

    os.str(string());
    os << setprecision(3) << 1.5 + (random() % 10000) / 20003.0;
    r.attributes.push_back (RecordAttr ("height", os.str()));

    return r;
}


