/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * pirsearch.cc
 * alex iliev, jan 2003
 * the main host-side program for querying the PIR server
 */

#include <iostream>

#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>

#include <sys/socket.h>
#include <sys/sccnet.h>
#include <netinet/in.h> /* for htons */

#ifdef _TIMING
#include <sys/time.h>
#include <time.h>
#endif

#include <common/sccutils.h>
#include <common/consts.h>



using namespace std;



static string query_card (const string& name,
			  struct sockaddr_scc * scc_server,
			  int sock)
{
    int rc;
    char buf[BUFSIZE];

    if ( (rc = sendto (sock, name.c_str(), name.length(),
		       0,
		       reinterpret_cast<struct sockaddr *>(scc_server),
		       sizeof (*scc_server)))
	 != static_cast<int>(name.length()) )
    {
	perror ("Sending request to card");
	close (sock);
	exit (EXIT_FAILURE);
    }

    socklen_t server_len = sizeof (*scc_server);
    if ( (rc = recvfrom (sock, buf, BUFSIZE-1,
			 0,
			 reinterpret_cast<struct sockaddr *>(scc_server),
			 &server_len))
	 < 0 )
    {
	perror ("Receiving reply from card");
	close (sock);
	exit (EXIT_FAILURE);
    }

    buf[rc] = '\0';
    return buf;
}



static void
usage(char *prog)
{
    fprintf(stderr, "Usage: %s cardno [-p port]\n",prog);
}



int main (int argc, char *argv[]) {

    int cardno = 1,		// the first card
	serverport = PIR_MAIN_PORT;

    struct sockaddr_scc scc_server;

    int s;			// the socket

    if ((argc < 2) ||(argc > 5)){
	usage(argv[0]);
	exit(0);
    }

    cardno = atoi (argv[1]);
    
    int i;
    while((i = getopt(argc, argv, "p:")) != EOF) {
	switch(i) {
	case('p'):
	    serverport = atoi(optarg); break;

	default:
	    usage(argv[0]);
	    exit(EXIT_FAILURE); break;
	} /* switch options */
    }

    if (cardno > 1 || serverport > 255) {
	fprintf(stderr,"Illegal Parameters: cardno !=0,1 or port > 255.\n");
	exit (EXIT_FAILURE);
    }

    
    // set up the socket to the card
    if ( scc_setup_send (&s, &scc_server, cardno, serverport) != 0 ) {
	perror ("Setting up send socket");
	exit (EXIT_FAILURE);
    }

    
    string buf;
    if (!getline (cin, buf)) {
	cerr << "Error reading request" << endl;
	exit(EXIT_FAILURE);
    }

    string answer = query_card (buf, &scc_server, s);
    cout << answer << endl;

    close (s);

}
